unit minimizer;

interface

uses classes, sysutils, minimizelib, formulan;


type

 TMinException = class(Exception)
 end;

 TMinMethod = (mmRosen, mmDavidon, mmFletcher, mmBroyden,
               mmReeves, mmPshenichny, mmSDescent, mmNewton);

 ppdouble = ^pdouble;

 TCallBack = procedure (a : double; var x : Array of double; n:integer) of Object;
 TOnEvaluateF = procedure (var x : array of double; var f:double) of Object;
 TOnEvaluateConstraint = procedure (var x : array of double; var f:double) of Object;
 TOnEvaluateGrad =  procedure (var x: array of double; var res:array of double; n:integer) of Object;
 TOnEvaluateHesse = procedure (var x : array of double; var res : ppdouble; n : integer) of Object;


 TMinimizer = class(TComponent)
 protected
  fsymbolic : boolean;
  fconstrained : boolean;
  fformula, fconsts, fvars : string;
  fvlist, fclist, fconstr : StringArray;
  fmaxiter : integer;
  fn : integer;
  ffun : TOnEvaluateF;
  fGrad : TOnEvaluateGrad;
  fHesse : TOnEvaluateHesse;
  fmethod : TMinMethod;
  fconstraint : TOnEvaluateConstraint;
  frestart : boolean;
  foniteration : TCallBack;
  feps1, feps2, fr, fk, fc : double;
  fvals : DoubleArray;
  fdllpath : string;
  fcconstr : array of string;
  procedure setC(newc : double);
  procedure SetMaxIter(x : integer);
  function GetMaxIter : integer;
  function GetIter : integer;
  procedure SetVars(v:string);
  procedure SetConsts(c:string);
  function GetConst(Index: string): double;
  procedure SetConst(Index: string; Value: double);
  function GetConstr(Index: integer): string;
  procedure SetConstr(Index: integer; Value: string);
  procedure SetFn(n : integer);
  procedure Init;
 public
  procedure Stop;
  procedure PrepareSymbolic(fmethod : TMinMethod; constrained : boolean = false);
  function F(var x : array of double) : double;
  function G(var x : array of double; index:integer) : double;
  procedure Grad(var x : array of double; var res : array of double);
  procedure Hesse(var x : array of double; res : ppdouble);
  function ConstraintCount : integer;
  procedure AddConstraint(Value: string);
  procedure DelConstraint(Index : integer);
  property Iteration : integer read GetIter;
  property ConstValue[Index: string]: double read GetConst write SetConst;
  property Constraint[Index: integer]: string read GetConstr write SetConstr;
  constructor Create(Owner: TComponent); override;
  destructor Free;
  function Minimize(var x : array of double) : double;
 published
  property DllPath : string read fdllpath write fdllpath;
  property Symbolic : boolean read fsymbolic write fsymbolic default false;
  property Constrained : boolean read fconstrained write fconstrained default false;
  property Restart : boolean read frestart write frestart default false;
  property Formula : string read fformula write fformula;
  property Vars : string read fvars write SetVars;
  property Consts : string read fconsts write SetConsts;
  property n : integer read fn write Setfn;
  property Method : TMinMethod read fmethod write fmethod default mmRosen;
  property MaxIteration : integer read GetMaxIter write SetMaxIter;
  property Eps1 : double read feps1 write feps1;
  property Eps2 : double read feps2 write feps2;
  property R : double read fr write fr;
  property C : double read fc write setC;
  property OnIteration : TCallBack read foniteration write fonIteration;
  property OnEvaluateF : TOnEvaluateF read ffun write ffun;
  property OnEvaluateGrad : TOnEvaluateGrad read fGrad write fGrad;
  property OnEvaluateHesse : TOnEvaluateHesse read fHesse write fHesse;
  property OnEvaluateConstraint : TOnEvaluateConstraint read fConstraint write fConstraint;
 end;

procedure Register;


implementation
uses math;

var
  cfun : string;
  cgrad : array of string;
  chesse : array of array of string;
  vals : DoubleArray;
  AF : TArtFormulaN;
  xn, xm : integer;
  M : TMinimizer;

procedure callback(a:double; x:pdouble; n:integer); cdecl; forward;


constructor TMinimizer.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  feps1 := DEF_PRECISION;
  feps2 := DEF_PRECISION;
  fr := 16;
  fc := 4;
  M := Self;
end;

destructor TMinimizer.Free;
begin
  SetMinimizeCallback(nil);
  fvals := nil;
  M := nil;
  DeInitDll;
  inherited Free;
end;

procedure TMinimizer.Init;
begin
  if MinimizeLib.DllHandle = 0 then
  begin
   InitDll(fdllpath);
   SetMinimizeCallback(callback);
  end;
end;

procedure TMinimizer.SetVars(v : String);
begin
 fvars := v;
 AF.VarList(v, fvlist);
end;

procedure TMinimizer.SetFn(n : integer);
begin
 fn := n;
 xn := n;
end;

procedure TMinimizer.Stop;
begin
 Init;
 StopMinimize;
end;

procedure TMinimizer.SetConsts(c : String);
begin
 fconsts := c;
 AF.VarList(c, fclist);
 if high(fclist) >= 0 then
  setlength(fvals,high(fclist)+1)
 else fvals := nil;
end;

procedure TMinimizer.setC(newc : double);
begin
 if newc <= 1 then raise TMinException.create('C should be greater than 1!');
end;


function TMinimizer.GetConst(Index: string): double;
var i:integer;
begin
 for i:=0 to high(fclist) do
  if fclist[i] = Index then
  begin
   result := fvals[i];
   exit;
  end;
 raise TMinException.create('Wrong constant name');
end;

procedure TMinimizer.SetConst(Index: string; Value: double);
var i:integer;
begin
 for i:=0 to high(fclist) do
  if fclist[i] = Index then
  begin
   fvals[i] := Value;
   exit;
  end;
 raise TMinException.create('Wrong constant name');
end;

function TMinimizer.GetConstr(Index: integer): string;
begin
   result := fconstr[index];
end;

procedure TMinimizer.SetConstr(Index: integer; Value: string);
begin
  fconstr[index] := Value;
end;

procedure TMinimizer.AddConstraint(Value: string);
begin
  setlength(fconstr, high(fconstr)+2);
  fconstr[high(fconstr)] := Value;
end;

procedure TMinimizer.DelConstraint(Index:integer);
var i:integer;
begin
  for i:=Index to high(fconstr)-1 do fconstr[i] := fconstr[i+1];
  if high(fconstr) = 0 then fconstr := nil
  else setlength(fconstr, high(fconstr)-1);
end;

function TMinimizer.ConstraintCount : integer;
begin
 result := length(fconstr);
end;


procedure TMinimizer.SetMaxIter(x : integer);
begin
 fmaxiter := x;
 try
  Init;
  SetMaxIteration(x);
 except
 end;
end;

function TMinimizer.GetMaxIter : integer;
begin
  try
   Init;
   result := GetMaxIteration;
  except
   result := fmaxiter;
  end;
end;

function TMinimizer.GetIter : integer;
begin
 Init;
 result := GetIteration;
end;

procedure callback(a:double; x:pdouble; n:integer); cdecl;
var xx : array of double absolute x;
begin
 with M do
 if @oniteration <> nil then
  oniteration(a, xx, n);
end;

function SymbolF(p : pdouble) : double; cdecl;
var x : array of double absolute p;
    i : integer;
begin
 for i := 0 to xn-1 do vals[xm+i] := x[i];
 AF.Compiled := cfun;
 result := AF.Compute(@vals);
end;

procedure SymbolG(fun : pfunction; px:pdouble; pres:pdouble; n:integer); cdecl;
var x : array of double absolute px;
    res : array of double absolute pres;
    i : integer;
begin
 for i := 0 to xn-1 do vals[xm+i] := x[i];
 for i := 0 to xn-1 do
 begin
  AF.Compiled := cgrad[i];
  res[i] := AF.Compute(@vals);
 end
end;

procedure SymbolH(fun : pfunction; px : pdouble; pres : ppdouble; n : integer); cdecl;
var x : array of double absolute px;
    res : array of array of double absolute pres;
    i,j : integer;
begin
 for i := 0 to xn-1 do vals[xm+i] := x[i];
 for i := 0 to xn-1 do
  for j := 0 to xn-1 do
  begin
   AF.Compiled := chesse[i,j];
   res[i,j] := AF.Compute(@vals);
  end;
end;

function NumericF(p : pdouble) : double; cdecl;
var x : array of double absolute p;
    c : double;
begin
 if M.Constrained then
 begin
  M.onevaluatef(x,result);
  M.OnEvaluateConstraint(x,c);
  result := result + M.fk * sqr(max(0,c));
 end
 else
  M.onevaluatef(x,result);
end;

procedure NumericG(fun : pfunction; px:pdouble; pres:pdouble; n:integer); cdecl;
var x : array of double absolute px;
    res : array of double absolute pres;
begin
 with M do
 if @fGrad = nil then MinimizeLib.Grad(NumericF,px,pres,n)
 else M.fGrad(x,res,n);
end;

procedure NumericH(fun : pfunction; px : pdouble; pres : ppdouble; n : integer); cdecl;
var x : array of double absolute px;
begin
 with M do
 if @fHesse = nil then MinimizeLib.Hesse(NumericF,px,Minimizelib.ppdouble(pres),n)
 else M.fHesse(x,pres,n);
end;

procedure TMinimizer.PrepareSymbolic(fmethod : TMinMethod; constrained : boolean = false);
var i,j:integer;
    s,f : string;
    c : char;
begin
  if fconsts <> '' then s := fconsts+','+fvars
  else s := fvars;

  f := fformula;
  if constrained and (length(fconstr)>0) then
  begin
   c := decimalseparator;
   decimalseparator := '.';
   f := f + '+'+floattostr(fk)+'*((max(0,'+fconstr[0]+'))^2';
   for i:=1 to high(fconstr) do f := f+'+(max(0,'+fconstr[i]+'))^2';
   f := f +')';
   decimalseparator := c;
  end;

  if length(fconstr) > 0 then
  begin
   setlength(fcconstr,length(fconstr));
  for i:=0 to high(fconstr) do
   fcconstr[i] := AF.compile(fconstr[i],s);
  end;

  cfun := AF.compile(f,s);
  if fmethod <> mmRosen then
  begin
   setlength(cgrad, fn);
   for i := 0 to fn-1 do
   begin
    AF.Compiled := cfun;
    cgrad[i] := AF.Compile(AF.Diff(fvlist[i]),s);
   end;
   if fmethod = mmNewton then
   begin
    setlength(chesse, fn, fn);
    for i := 0 to fn-1 do
    begin
     for j := 0 to fn-1 do
     begin
      AF.Compiled := cgrad[i];
      chesse[i,j] := AF.Compile(AF.Diff(fvlist[j]),s);
     end;
    end;
   end;
  end;
  xm := high(fvals)+1;
  setlength(vals,xm+fn);
end;

function TMinimizer.Minimize(var x : array of double) : double;
var i:integer;
    met : pMethod;
    x_, y_ : pdouble;
    y : array of double;

function eq(var x : array of double; var y : array of double):boolean;
var d : double;
    i : integer;
begin
 d := 0;
 for i:=0 to high(x) do d := d + sqr(x[i]-y[i]);
 result := sqrt(d) < feps1/2;
end;
begin
 Init;
 SetMaxIteration(fmaxiter);
 x_ := @(x[0]);
 setlength(y,length(x));
 y_ := @(y[0]);


 case method of
  mmRosen: met := Rosen;
  mmDavidon: met := Davidon;
  mmFletcher: met := Fletcher;
  mmBroyden: met := Broyden;
  mmReeves: met := Reeves;
  mmPshenichny: met := Pshenichny;
  mmSDescent: met := SDescent;
  mmNewton: met := Newton;
 end;

 M := Self;
 AllowMinimize;

 if fsymbolic then
 begin
  SetGrad(@SymbolG);
  SetHesse(@SymbolH);

  if fconstrained then
  begin
   fk := fr;
   met := Rosen;
   PrepareSymbolic(mmRosen, true);
   for i := 0 to high(fvals) do vals[i] := fvals[i];
   result := met(@SymbolF,x_,fn,feps1,feps2);

   repeat
    fk := fk * fc;
    PrepareSymbolic(mmRosen, true);
    for i:=0 to high(x) do y[i] := x[i];
    result := met(@SymbolF,x_,fn,feps1,feps2)
   until eq(x,y);
   PrepareSymbolic(mmRosen);
   result := M.F(x);
  end
  else
  begin
   PrepareSymbolic(method);
   for i := 0 to high(fvals) do vals[i] := fvals[i];
   if frestart then
     result := Minimizelib.Restart(met,@SymbolF,x_,fn,feps1,feps2)
   else
     result := met(@SymbolF,x_,fn,feps1,feps2)
  end;
 end
 else
 begin
  SetGrad(@NumericG);
  SetHesse(@NumericH);
  if fconstrained then
  begin
   met := Rosen;
   fk := fr;
   result := met(@NumericF,x_,fn,feps1,feps2);
   repeat
    fk := fk * fc;
    for i:=0 to high(x) do y[i] := x[i];
    result := met(@NumericF,x_,fn,feps1,feps2)
   until eq(x,y);
   fconstrained := false;
   result := M.F(x);
   fconstrained := true;
  end
  else
   if frestart then
     result := Minimizelib.Restart(met,@NumericF,x_,fn,feps1,feps2)
   else
     result := met(@NumericF,x_,fn,feps1,feps2);
 end;
end;

function TMinimizer.F(var x : array of double) : double;
var x_ : pdouble;
begin
 x_ := @(x[0]);
 if fsymbolic then
  result := SymbolF(x_)
 else
  onevaluatef(x,result);
end;

function TMinimizer.G(var x : array of double; index:integer) : double;
var
    i : integer;
begin
 if fsymbolic then
 begin
  for i := 0 to xn-1 do vals[xm+i] := x[i];
  AF.Compiled := fcconstr[index];
  result := AF.Compute(@vals);
 end
 else
  onevaluateconstraint(x,result);
end;


procedure TMinimizer.Grad(var x : array of double;var res : array of double);
var x_, res_ : pdouble;
begin
 x_ := @(x[0]);
 res_ := @(res[0]);
 if fsymbolic then
  SymbolG(nil,x_,res_,fn)
 else
  NumericG(nil,x_,res_,fn);
end;

procedure TMinimizer.Hesse(var x : array of double; res : ppdouble);
var x_ : pdouble;
begin
 x_ := @(x[0]);
 if fsymbolic then
  SymbolH(nil,x_,res,fn)
 else
  NumericH(nil,x_,res,fn);
end;

procedure Register;
begin
  RegisterComponents('Art', [TMinimizer]);
end;


initialization
  AF := TArtFormulaN.Create(nil);

finalization
  AF.Free;
  cgrad := nil;
  for xn:=0 to high(chesse) do chesse[xn] := nil;
  chesse := nil;
  vals := nil;

end.
